import java.io.*;
import java.awt.*;

/**
 * A Circular List or list that keep vertices on the polygon
 * in clockwise order. Vertices could add between two vertices:
 * usually on the line.
 * 
 * Size is at least three and never decrease until finalized.
 */
public class CircularList
{
    
    private ListNode   first;   // server as the start point
    private ListNode   last;    // useful during initialization
    private ListNode   current;  // used to traversal the list
    
    
    private int         size;   // total size
    private boolean     isEnd;
      
    /**
     * Constructor , 
     * assume the ploygon has at least three vertices
     */
    public CircularList(Object [] obj, boolean isClockwise){
        size=obj.length ;

        // create a circular linked list queue
        first=last= new ListNode();
        if(isClockwise){
            for(int i=0;i<size-1;i++){
                last.setData(obj[i]);
                last.setNextNode(new ListNode());
                last=last.getNextNode();
            }
            last.setData(obj[size-1]);
        } else {
            for(int i=size-1;i>0;i--){
                last.setData(obj[i]);
                last.setNextNode(new ListNode());
                last=last.getNextNode();
            }
            last.setData(obj[0]);
        }
        last.setNextNode(first);
        isEnd = false;
    }
    
    /**
     * Reset the start point, The start point - obj
     * on the clipping polygon: outside of polygon
     * Make sure the obj is in the list!!!, otherwise infinite
     * loop
     */
    public void resetStartingObject(Object obj){
        while (last.getNextNode().getData()!=obj){
            last=last.getNextNode();
        }
        first=last.getNextNode();
    }
    
    /**
     * Same as resetStartingObject(Object obj) except to
     * current object in the list
     */
    public void resetStartingToCurrent(){
        while(last.getNextNode()!=current){
            last=last.getNextNode();
        }
        first = last.getNextNode();
    }
    
    /**
     * Go to the start point, initialize iteration
     */
    public void goToTheBeginning(){
        current = first;
        isEnd = false;
    }
    
    /**
     * Go to the next node
     */
    public boolean goToTheNext(){
        if(isEnd == false) {
            current = current.getNextNode();
            if(current==first){
                isEnd = true;
                return false;
            }
            return true;
        }
        return false;
    }
    
    /**
     * Go To the next node anyway
     */
    public void forceToNext(){
        current = current.getNextNode();
    }
    
    /**
     * Set current position: make sure ListNode is in the list.
     * Used for switching points on the intersection
     */
    public void setCurrentPosition(ListNode node){
        current = node;
        isEnd = false;
    }
    
    /**
     * Get current Positio
     */
    public ListNode getCurrentPosition(){
        return current;
    }
    
    /**
     * Get current & next obj: first point on the current line segment
     */
    public Object getCurrentObj(){
        return current.getData();
    }
    public Object getTheNextObj(){
        return current.getNextNode().getData();
    }
    
    /**
     * Insert after current node
     * Special condition: current is on the last
     */
    public void insertAfterCurrent(Object obj){
        size++;
        ListNode temp = new ListNode();
        temp.setData(obj);
        temp.setNextNode(current.getNextNode());
        current.setNextNode(temp);
        if(current==last) {
            last = temp;
        }
    }
    
    /**
     * Insert to the End
     */
    public void insertAfterTheLast(Object obj){
        size++;
        ListNode temp = new ListNode ();
        temp.setData(obj);
        temp.setNextNode(first);
        last.setNextNode(temp);
        last = temp;
    }
    
    boolean isEndOfList(){  return isEnd; }
    
    public int getSize(){ return size; }
    
    /**
     * Help free the memory along the circular list before
     * garbage collection taking place
     */
    public void finilize (){
        for(int i=0;i<size-1;i++){
            last.setNextNode(null);
            last.setData(null);
            last=first;
            first = first.getNextNode();
        }
        first.setNextNode(null);
        first.setData(null);
        current = first = last = null;
    }
    // test driver for the circular list
    public static void main(String [] args){
        
        Point [] points = new Point [5];
        for (int i=0;i<5;i++){
            points[i] = new Point(i*i,i*i);
        }
        
        CircularList alist = new CircularList (points, false );
        alist.resetStartingObject (points[0]);
        alist.goToTheBeginning ();
        
        Point apoint, bpoint, cpoint;
        while(!alist.isEndOfList ()){
            apoint = (Point) alist.getCurrentObj ();
            bpoint = (Point) alist.getTheNextObj ();
            cpoint = new Point( (apoint.x+bpoint.x)/2, (apoint.y+bpoint.y)/2);
            alist.insertAfterCurrent (cpoint);
            alist.goToTheNext ();
            alist.goToTheNext ();
        }
        // alist.insertAfterTheLast(new Point(100,100));
        
        alist.goToTheBeginning ();
        for(int i=0;i<alist.getSize();i++){
            cpoint = (Point) alist.getCurrentObj ();
            System.out.println("Point "+i+": "+ cpoint.x+", "+cpoint.y);
            alist.goToTheNext();
        }
    }
 }
   
