/**
 * A simple version of clipping algorithm for polygons to be used as for cmsc427 homework #3
 * 
 * @author Hua Guo
 * @date Feb. 17, 2000
 */

import java.awt.*;
import java.awt.event.*;

/**
 * This is the drawing panel that we use to draw on. We must create this class
 * extending Panel so that we can override the crucial 'paint' method which
 * gets called whenever the system determines that Panel needs to be redrawn.
 * We also define simple event handlers to determine when the user clicks
 * on the drawing panel.
 */
class DrawingPanel extends Panel
{
    private Point [] source = null;         // the source polygon
    private Point [] clipping = null;       // the clipping polygon
    private Button button;        // the next button
    private PolygonReader reader;       // for repeat readings
    private boolean iseof = false;
      
    /**
     * Simple constructor: adds a button listener to this panel
     * The button values: goNext and reStart
     */
    public DrawingPanel() {
        button = new Button("Go Next");
        button.setForeground (Color.yellow);
        button.addActionListener(new DrawingActionListener (this));
        setClickButton(button);
        add(button);
        
        reader = new PolygonReader ("hw3-data.txt");
        repaint();
    }
    
    /**
     * Bind button with current panel
     */
    public void setClickButton(Button button){
        this.button = button;
    }
    
    /**
     * The main paint function. 
     * 
     * Read source polygon and clipping polygon from the
     * input file: hw3-data.txt. The format of polygon could be either clockwise
     * or anticlockwise.
     * 
     * Using the Weiler-Atherton algorithm to render clipping area:
     * problem haven't been solved: 1) polygons share linesegments
     * 2) vertex on one polygon is on another polygon. 
     * 
     * Situation solved: 1) lines strickly intersect with each other or not,
     * 2) one is inside of another, 3) source polygon is outside of clipping polygon
     */
    public void paint(Graphics g){
        if(iseof){  // EOF of input file; reset button
            button.setForeground (Color.yellow );
            button.setLabel ("Go Next");
            reader = new PolygonReader ("hw3-data.txt");
            iseof = false;
        }
        
        // reading polygon
        source = reader.FetchNextPolygon (PolygonReader.SOURCEPOLYGON );
        clipping = reader.FetchNextPolygon (PolygonReader.CLIPPINGPOLYGON );
        int [] xpoints1 = null;
        int [] ypoints1 = null;
        int [] xpoints2 = null;
        int [] ypoints2 = null;
        
        if(source!=null && clipping!=null){
            // initialize the working data structure ClippingVirtualPanel,
            // the polygons are stored in clockwise order, even the input 
            // doesn't
            ClippingVirtualPanel cvPanel = 
                         new ClippingVirtualPanel(source, clipping);
            
            // find all the intersections and prepare the data structure
            // for renderring
            cvPanel.findIntersections();
            
            // rendering
            g.setColor(Color.red);
            Polygon smallpolygon= cvPanel.fetchNextPolygon ();
            while(smallpolygon != null){
                g.fillPolygon(smallpolygon);
                smallpolygon= cvPanel.fetchNextPolygon ();
            }
            
            xpoints1 = new int [source.length];
            ypoints1 = new int [source.length];
            for(int i=0;i<source.length;i++){
                xpoints1[i]=source[i].x;
                ypoints1[i]=source[i].y;
            }
            xpoints2 = new int [clipping.length];
            ypoints2 = new int [clipping.length];
            for(int i=0;i<clipping.length;i++){
                xpoints2[i]=clipping[i].x;
                ypoints2[i]=clipping[i].y;
            }
            
            // outline the source polygon (blue) and clipping polygon (black)
            g.setColor (Color.blue);
            g.drawPolygon (xpoints1, ypoints1, source.length);
            g.setColor (Color.black);
            g.drawPolygon (xpoints2, ypoints2, clipping.length);
            
        }
        else {
            iseof = true;
            button.setForeground (Color.green );
            button.setLabel("Restart");
        }
    }
}

