// Quicksort, modified from:
// Source code example for "A Practical Introduction
// to Data Structures and Algorithm Analysis"
// by Clifford A. Shaffer, Prentice Hall, 1998.
// Copyright 1998 by Clifford A. Shaffer

// Sorting main function for testing correctness of sort algorithm.
// To use: <sortname> [+/-] <size_of_array> <threshold>
//  + means increasing values, - means decreasing value and no
//    parameter means random values;
// where <size_of_array> controls the number of objects to be sorted;
// and <threshold> controls the threshold parameter for certain sorts, e.g.,
//   cutoff point for quicksort sublists.

// slightly modified to sort index according floating point keys.

import java.util.*;

public class Sort {

    static int THRESHOLD = 0;

    static int ARRAYSIZE = 20;

    // assume array.length == key.length
    static void sort(int[] array, float[] key) {
        qsort(array, key, 0, key.length-1);
    }

    static int MAXSTACKSIZE = 1000;

    static void qsort(int[] array, float[] key,  int i, int j) { // Quicksort
        int pivotindex = findpivot(key, i, j);      // Pick a pivot
        swap(array, key, pivotindex, j);    // Stick pivot at end
        // k will be the first position in the right subarray
        int k = partition(array, key, i-1, j, key[j]);
        swap(array, key, k, j);             // Put pivot in place
        if ((k-i) > 1) qsort(array, key, i, k-1);  // Sort left partition
        if ((j-k) > 1) qsort(array, key, k+1, j);  // Sort right partition
    }

    static int partition(int[] array, float[] key, int l, int r, float pivot) {
        do {                // Move the bounds inward until they meet
            while (key[++l] < pivot); // Move left bound right
            while ((r!=0) && (key[--r]>pivot)); // Move right bound
            swap(array, key, l, r);         // Swap out-of-place values
        } while (l < r);                    // Stop when they cross
        swap(array, key, l, r);           // Reverse last, wasted swap
        return l;           // Return first position in right partition
    }
      // Swap two objects in an array
    public static void swap(int[] array, float[] key, int p1, int p2) {
        int temp = array[p1];
        float tempk = key[p1];
        array[p1] = array[p2];
        array[p2] = temp;
        key[p1] = key[p2];
        key[p2] = tempk;
    }

  static private Random value = new Random(); // Hold the Random class object

    static int random(int n) { // My own function
        return Math.abs(value.nextInt()) % n;
    }

    static int findpivot(float[] key, int i, int j)
    { return (i+j)/2; }

// Main routine for sorting class driver
// This is the version for running timings
    public static void main(String args[]) {

        int i;
        int input = 0;
        int currarg = 0;
        
        if(args.length>0){
            ARRAYSIZE = Integer.parseInt (args[0]);
        }

        float[] keys = new float[ARRAYSIZE];
        int[] array = new int[ARRAYSIZE];

  System.out.println("Input: " + input +
          ", size: " + ARRAYSIZE + ", threshold: " + THRESHOLD);


  for (i=0; i<ARRAYSIZE; i++){
      keys[i] = (float)random(ARRAYSIZE*2);  // Random
      array[i]=i;
  }
  for (i=1; i<ARRAYSIZE; i++)
      System.out.print(array[i] + " ");
  System.out.println("Original arr");
  for (i=1; i<ARRAYSIZE; i++)
      System.out.print((int)keys[i] + " ");
  System.out.println("Original key");

  long time1 = System.currentTimeMillis();
  sort(array,keys);
  long time2 = System.currentTimeMillis();
  System.out.println("Time is " + (time2-time1));
  
  System.out.println("Done sorting");

  // Double-check that the sort was correct
  for (i=1; i<ARRAYSIZE; i++)
      System.out.print(array[i] + " ");
  System.out.println("");
  for (i=1; i<ARRAYSIZE; i++)
      System.out.print((int)keys[i] + " ");
  System.out.println("Done");
    }
}

