/**
 * This is code that implements a method to determine if a point
 * is inside of a polygon.
 * Plus: calculate line intersection
 */
import java.awt.*;

/**
 * Includes, 1) test whether a point is inside of a polygon or not: pointInPolygon
 *  2) test whether a polygon is clockwise or not: isClockwisePolygon
 *  3) test whather two line segments intersect or not: isTwoLineIntersect
 */
public class Util{
    /**
     * A bit of test code to show use of the pointInPolygon method.
     * Test: pointInPolygon, isTwoLineIntersection, isClockWisePolygon
     */
    static public void main (String[] args) {
	Point vertices[] = new Point[4];
	Point p = new Point();

	vertices[0] = new Point(0, 0);
	vertices[1] = new Point(300, 300);
	vertices[2] = new Point(100, 0);
    vertices[3] = new Point(0,100);
	p.setLocation(13, 3);
	System.out.println("point in polygon: " + pointInPolygon(p, vertices));
    System.out.println("Is clockwise? " + isClockwisePolygon(vertices));
    
    float [] t = new float[2];
    System.out.println(t[0]);
          System.out.println(t[1]);
  
    
    if(isTwoLineIntersect(vertices[0], vertices[1], vertices[2], vertices[3], t)){
        System.out.println("Yes. Intesect! ");
        System.out.println(t[0]);
        System.out.println(t[1]);
    }
    
    }

    /**
     * Solve intersection
     * @param P1 and P2 - the first line segment
     * @param P3 and P4 - the second line segment
     * @param t1 the ratio of intesection points on both line segments, float [2]
     * @return true if P1 and P2 line segments intesect with each other
     */
    public static boolean isTwoLineIntersect(Point p1, Point p2, Point p3, Point p4,
                                            float [] t){
        // assume first line segments do not overlap 
        // also assume p1!=p2 and p3!=p4, they are distiguishable
        
        // return false if lines are parallel, first case: all vertical line
        if(p1.x==p2.x && p3.x==p4.x) return false;
        // second case: same slope
        else if((p2.x-p1.x)*(p4.y-p3.y) == (p2.y-p1.y)*(p4.x-p3.x)) return false;
        
        // if lines are parallel and overlapping ====== real preblem!!!!
        
        int a1 = p2.x-p1.x;
        int b1 = p3.x-p4.x;
        int c1 = p3.x-p1.x;
        int a2 = p2.y-p1.y;
        int b2 = p3.y-p4.y;
        int c2 = p3.y-p1.y;
        // denomiator = a1*b2 - b1*a2
        float denominator = a1*b2 - b1*a2;
        // t[0] = (c1*b2 - c2*b1)/denominator
        t[0] = ((float)(c1*b2 - c2*b1))/denominator;
        if(t[0] < 0 || t[0] > 1) return false;
        
        // t[1] = (c2*a1 - c1*a2)/denominator
        t[1] = ((float)(c2*a1 - c1*a2))/denominator;
        if(t[1] < 0 || t[1] > 1) return false;
        
        return true;
    }
                                            
    
    /**
     * Returns true if point, is inside polygon.
     */
    public static boolean pointInPolygon(Point p, Point vertices[]) {
	int i;
	float angle = 0.0f;
	boolean inside;

				// Implemented with the winding angle algorithm.
				// Just add up the angles between points
	for (i=0; i<vertices.length-1; i++) {
	    angle += angleBetweenPoints(vertices[i], p, vertices[i+1]);
	}
	angle += angleBetweenPoints(vertices[vertices.length-1], p, vertices[0]);

				// Allow for a bit of rounding
				// Ideally, angle should be 2*pi.
	if (Math.abs(angle) > 6.2) {
	    inside = true;
	} else {
	    inside = false;
	}
	return(inside);
    }
    
    /**
     * Returns true if current polygon is Clockwise
     */
    public static boolean isClockwisePolygon(Point vertices[]){
        boolean clockwise = false;
    	float angle = 0.0f;
        Point center;

        for(int i=0;i<vertices.length-2;i++){
            center = new Point ((vertices[i].x + vertices[i+1].x + vertices[i+2].x)/3,
                            (vertices[i].y + vertices[i+1].y + vertices[i+2].y)/3);
            if(pointInPolygon(center, vertices)){
	            for (int j=0; j<vertices.length-1; j++) {
	                angle += angleBetweenPoints(vertices[j], center, vertices[j+1]);
	            }
	            angle += angleBetweenPoints(vertices[vertices.length-1], 
                                            center, vertices[0]);
                if(angle > 0.){
                    clockwise = false;
                } else {
                    clockwise = true;
                }
                break;
            }
        }
        return clockwise;
    }

    /**
     * Returns the angle at p from p1 to p to p2.
     * (Positive is counter-clockwise).
     */
    public static float angleBetweenPoints(Point p1, Point p, Point p2) {
	Point v1 = new Point();
	Point v2 = new Point();
	float z, s;
	float theta;
	float t1, t2;

	v1.setLocation(p1.x - p.x, p1.y - p.y);
	v2.setLocation(p2.x - p.x, p2.y - p.y);

	z = (v1.x * v2.y) - (v1.y * v2.x);
	s = (z >= 0.0f) ? 1.0f : -1.0f;
	t1 = length(v1) * length(v2);
	if (t1 == 0.0f) {
	    theta = 0.0f;
	} else {
	    t2 = ((v1.x * v2.x) + (v1.y * v2.y)) / t1;
	    if ((t2 < -1) || (t2 > 1)) {
		theta = 0.0f;
	    } else {
		theta = (float)(s * Math.acos(t2));
	    }
	}
    
	return theta;
    }

    public static float length(Point v) {
	return (float)Math.sqrt(v.x*v.x + v.y*v.y);
    }
    
    /**
     * return the intersection of two line segments which closer to p1, 
     * return null if no intersection
     */
    public static Point calculateIntersection (Point p1, Point p2, 
                                               Point p3, Point p4){
        return new Point();
    }
}
